      program make_data
      implicit none

c     Purpose: to generate a simple test "atmos_data.txt" input file for KARINE.

      include './max.inc'
      include './param.inc'
      integer i,j,band,quad,m,Nb,Nq
      integer t,res
      double precision lambda_lo(1:Nbmx),lambda_hi(1:Nbmx)
      double precision flux_sol(1:Nbmx),flux_esp(1:Nbmx)
      double precision k_gas(1:Nbmx,1:Nmax,Nqmx)
      double precision ka_gas(1:Nbmx,1:Nmax,Nqmx)
      double precision kd_gas(1:Nbmx,1:Nmax,Nqmx)
      double precision w_gas(1:Nbmx,1:Nmax,Nqmx)
      double precision g_gas(1:Nbmx,1:Nmax,Nqmx)
      double precision weight(1:Nbmx,Nqmx),e(0:Nmax)
      double precision alt(0:Nmax),pres(0:Nmax),temp(0:Nmax),Tsol,Tesp
      double precision dens(0:Nmax),adia(0:Nmax)
      double precision grav(0:Nmax),zeta(0:Nmax)
      double precision capa(0:Nmax)
      double precision em_sol(1:Nbmx),em_esp(1:Nbmx)
      double precision k_aero(1:Nbmx,1:Nmax)
      double precision ka_aero(1:Nbmx,1:Nmax)
      double precision kd_aero(1:Nbmx,1:Nmax)
      double precision g_aero(1:Nbmx,1:Nmax)
      double precision w_aero(1:Nbmx,1:Nmax)
      double precision k_cloud(1:Nbmx,1:Nmax)
      double precision ka_cloud(1:Nbmx,1:Nmax)
      double precision kd_cloud(1:Nbmx,1:Nmax)
      double precision g_cloud(1:Nbmx,1:Nmax)
      double precision w_cloud(1:Nbmx,1:Nmax)
      double precision Tref
      double precision dat,ep_maille,wsum

c     ================================
c     Beginning of the data input section
c     ================================
c     Feel free to modify the data specified in this section for your model atmosphere.
c     This program will check the specified data for errors and inconsistencies before
c     writing the "atmos_data.txt" file. Do not ignore error messages if any.
c     Please remember the correspondance between data and analysis grids (see below).

c     Correspondance between data (alt) and analysis (e) grids :
c     
c     
c                                     Tesp
c               \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
c     e(m)   ------------------- temp(m-1) ------------------- alt(m-1)
c     
c     e(m-1) ------------------------------------------------------
c     
c                ------------------- temp(m-2) ------------------- alt(m-2)
c     
c     e(m-2) ------------------------------------------------------
c
c                                       .
c                                       .
c                                       .
c     
c     e(3) ------------------------------------------------------
c     
c             ------------------- temp(2) ------------------- alt(2)
c     
c     e(2) ------------------------------------------------------
c     
c             ------------------- temp(1) ------------------- alt(1)
c     
c     e(1) ------------------------------------------------------
c
c     e(0)  ------------------- temp(0) ------------------- alt(0)
c             //////////////////////////////////////////////////////////
c                                    Tsol


c
c     Tip: do not forget the ".D+0" when entering double-precision data;
c     otherwise you may encounter numerical precision issues.
c


c     ---------------------------------------------------------------------------------------------------------------------
c     vertical and spectral grids - temperatures and various atmospheric data
c     ---------------------------------------------------------------------------------------------------------------------

c     m is the number of atmospheric gas layers
      m=4
c     Nb is the number of narrowband spectral intervals
      Nb=1
c     Nq is the quadrature order
      Nq=1


      ep_maille=1.0D+0 ! m
c     alt(i), i=0,m-1 are altitudes (in meters) of the data grid
c     the following lines can be modified
      do i=0,m-1
         alt(i)=ep_maille*i
      enddo

c     e(i), i=0,m are altitudes (in meters) of the analysis grid
c     use the following line codes (do not modify !)
      e(0)=alt(0)
      do i=1,m-1
         e(i)=(alt(i)+alt(i-1))/2.0D+0
      enddo
      e(m)=alt(m-1)

      Tref=0.0D+0 ! K
c     temp(i), i=0,m-1 are the temperatures (in kelvin) IN THE GAS at corresponding altitude levels alt(i)
      temp(0)=300.0D+0
      temp(1)=280.0D+0
      temp(2)=250.0D+0
      temp(3)=240.0D+0
      temp(4)=200.0D+0
c     Tsol is the ground temperature (in kelvin). Possible discontinuity between Tsol and temp(0)
      Tsol=320.0D+0 ! K
c     Tesp is the temperature of space (in kelvin). Possible discontinuity between Tesp and temp(m-1)
      Tesp=100.0D+0

c     + pres(i), i=0,m-1 are the pressures (pascal) at corresponding altitude levels alt(i)
      pres(0)=101300.D+0
      pres(1)=99000.D+0
      pres(2)=56000.D+0
      pres(3)=5600.D+0
      pres(4)=10.D+0

c     + dens(i), i=0,m-1 are the atmospheric densities (kilogram per cubic meter) at corresponding altitude levels alt(i)
c     + adia(i), i=0,m-1 are the adiabatic lapse rates (kelvin per kilometer) at corresponding altitude levels alt(i)
c     + grav(i), i=0,m-1 are the accelerations of gravity (meters per square seconds) at corresponding altitude levels alt(i)
c     + zeta(i), i=0,m-1 are the non-ideality factors (unitless) at corresponding altitude levels alt(i) : P.V=zeta.n.r.T
c     + capa(i), i=0,m-1 are the specific heats (joule per kilogram per kelvin) at corresponding altitude levels alt(i)     
      do i=0,m-1
         dens(i)=1.0D+0
         adia(i)=1.0D+0
         grav(i)=1.0D+0
         zeta(i)=1.0D+0
         capa(i)=1.0D+0
      enddo

c     lambda_lo(band), band=1,Nb (micrometers) are the lower limits of each spectral interval.
c     lambda_hi(band), band=1,Nb (micrometers) are the higher limits of each spectral interval.
      lambda_lo(1)=1.7D+0
      lambda_hi(1)=1.8D+0
c      lambda_lo(2)=1.8D+0
c      lambda_hi(2)=1.9D+0
c     em_sol(band), band=1,Nb are the ground emissivities in each spectral invertal
      em_sol(1)=0.9D+0
c      em_sol(2)=1.0D+0
c      em_sol(1)=0.75D+0
c      em_sol(2)=0.75D+0
c     em_esp(band), band=1,Nb are the emissivities of space in each spectral invertal
      em_esp(1)=1.D+0
c      em_esp(2)=1.D+0
c     flux_sol(band) and flux_esp(band) are respectively the input fluxes at ground level and space level,
c     per narrowband, if anything else than Planck must be used as input fluxes.
      flux_sol(1)=0.0D+0
      flux_esp(1)=1.0D+0/(lambda_hi(1)-lambda_lo(1))
c      flux_sol(2)=1.0D+2
c      flux_esp(2)=0.0D+0

c     weight(band,quad), quad=1,Nb are the quadrature weights of spectral interval index 'band'.
      do band=1,Nb
            weight(band,1)=1.0D+0
c            weight(band,2)=0.1D+0
c            weight(band,3)=0.1D+0
      enddo
c     warning: the sum of quadrature weights for each spectral interval must be equal to 1.
      



c     ---------------------------------------------------------------------------------------------------------------------
c     GAS spectroscopic data
c     ---------------------------------------------------------------------------------------------------------------------

c     ka_gas(band,i,quad) are the gas absorbtion coefficients (inverse meters)
c     kd_gas(band,i,quad) are the gas scattering coefficients (inverse meters)
c     g_gas(band,i,quad) are the gas asymetry parameters
c     + for the spectral interval index 'band'
c     + for the atmospheric layer index 'i'
c     + for the quadrature index 'quad'
      do i=1,m
         ka_cloud(1,i)=1.0D+0
         kd_cloud(1,i)=0.0D+0
         g_cloud(1,i)=0.0D+0
      enddo


c     ---------------------------------------------------------------------------------------------------------------------
c     CLOUDS and AEROSOLS spectroscopic data
c     ---------------------------------------------------------------------------------------------------------------------

      do band=1,Nb
         do i=1,m
            do quad=1,Nq
               ka_gas(band,i,quad)=1.0D-12
               kd_gas(band,i,quad)=1.0D-12
               g_gas(band,i,quad)=1.0D-12
            enddo ! quad
            ka_aero(band,i)=1.0D-12
            kd_aero(band,i)=1.0D-12
            g_aero(band,i)=1.0D-12
         enddo
      enddo

c     I wrote this program keeping in mind it has to be easy to read and modify by
c     any potential KARINE user. However, you need to modifiy spectral properties in
c     the program itself, compile and execute it each time you want to generate a
c     KARINE input data file.
c     If you intend to use this program on a regular basis, you may be interested by
c     using an input .txt file where all the properties of your atmosphere are defined.
c     You will no longer need to compile the program each time you want to produce
c     a different "atmos_data.txt" file.
c
c     ================================
c     End of the data input section
c     ================================
c


c     Computation of total extinction coefficients and single-scattering albedoes
      do band=1,Nb
         do i=1,m
            do quad=1,Nq
               k_gas(band,i,quad)= ka_gas(band,i,quad)+ 
     &              kd_gas(band,i,quad)
               w_gas(band,i,quad)=kd_gas(band,i,quad)/
     &              k_gas(band,i,quad)
            enddo ! quad
            k_cloud(band,i)= ka_cloud(band,i)+ 
     &           kd_cloud(band,i)
c
c     Commented because of the division by 0. Instead, w is set to 0.
c            w_cloud(band,i)=kd_cloud(band,i)/
c     &           k_cloud(band,i)
            w_cloud(band,i)=0.0D+0
c
            k_aero(band,i)= ka_aero(band,i)+ 
     &           kd_aero(band,i)
c
c     Commented because of the division by 0. Instead, w is set to 0.
c            w_aero(band,i)=kd_aero(band,i)/
c     &           k_aero(band,i)
            w_aero(band,i)=0.0D+0
c
         enddo
      enddo
      t=0 ! mean no error found at this stage

c     Checking quadrature weights
      do band=1,Nb
         wsum=0.0D+0
         do quad=1,Nq
            wsum=wsum+weight(band,quad)
         enddo ! quad
         if (wsum.ne.1.0D+0) then
            write(*,*) 'Sum of quadrature weights for band',
     &           band,' =',wsum
            t=1
            goto 666
c     Typically, if you get this error, it's most likely because you did
c     not use the ".D+0" for double-precision data specifications
c     (or maybe the sum is really not equal to 1 !).
         endif
      enddo ! band

c     Checking for NaN
      do band=1,Nb
         do i=1,m
            do quad=1,Nq
               dat=k_gas(band,i,quad)
               call test_nan(dat,res)
               if (res.eq.1) then
                  write(*,*) 'k_gas(',band,',',i,',',quad,')=',dat
                  t=1
                  goto 12345
               endif
               dat=ka_gas(band,i,quad)
               call test_nan(dat,res)
               if (res.eq.1) then
                  write(*,*) 'ka_gas(',band,',',i,',',quad,')=',dat
                  t=1
                  goto 12345
               endif
               dat=kd_gas(band,i,quad)
               call test_nan(dat,res)
               if (res.eq.1) then
                  write(*,*) 'kd_gas(',band,',',i,',',quad,')=',dat
                  t=1
                  goto 12345
               endif
               dat=w_gas(band,i,quad)
               call test_nan(dat,res)
               if (res.eq.1) then
                  write(*,*) 'w_gas(',band,',',i,',',quad,')=',dat
                  t=1
                  goto 12345
               endif
               dat=g_gas(band,i,quad)
               call test_nan(dat,res)
               if (res.eq.1) then
                  write(*,*) 'g_gas(',band,',',i,',',quad,')=',dat
                  t=1
                  goto 12345
               endif
            enddo ! quad
            
            dat=k_aero(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'k_aero(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=ka_aero(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'ka_aero(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=kd_aero(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'kd_aero(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=w_aero(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'w_aero(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=g_aero(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'g_aero(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            
            dat=k_cloud(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'k_cloud(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=ka_cloud(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'ka_cloud(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=kd_cloud(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'kd_cloud(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=w_cloud(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'w_cloud(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            dat=g_cloud(band,i)
            call test_nan(dat,res)
            if (res.eq.1) then
               write(*,*) 'g_cloud(',band,',',i,')=',dat
               t=1
               goto 12345
            endif
            
         enddo ! i
      enddo ! band

12345 continue
      if (t.eq.1) then
         write(*,*) 'NaN have been detected'
         goto 666
      endif

c     Production of input "atmos_data.txt" file.
      open(10,file='./atmos_data.txt')
      write(10,*) m,Nb,Nq
      write(10,'(1000e17.9)') (alt(i),i=0,m-1)
      write(10,'(1000e17.9)') (e(i),i=0,m)
      write(10,'(1000e17.9)') (pres(i),i=0,m-1)
      write(10,'(1000e17.9)') (temp(i),i=0,m-1),Tsol,Tesp
      write(10,'(1000e17.9)') (dens(i),i=0,m-1)
      write(10,'(1000e17.9)') (adia(i),i=0,m-1)
      write(10,'(1000e17.9)') (grav(i),i=0,m-1)
      write(10,'(1000e17.9)') (zeta(i),i=0,m-1)
      write(10,'(1000e17.9)') (capa(i),i=0,m-1)
      do band=1,Nb
         write(10,'(1000e17.9)') lambda_lo(band),lambda_hi(band),
     &        em_sol(band),em_esp(band),
     &        (weight(band,quad),quad=1,Nq),
     &        flux_sol(band),flux_esp(band)
         do quad=1,Nq
            write(10,'(1000e17.9)') (k_gas(band,i,quad),i=1,m)
            write(10,'(1000e17.9)') (ka_gas(band,i,quad),i=1,m)
            write(10,'(1000e17.9)') (kd_gas(band,i,quad),i=1,m)
            write(10,'(1000e17.9)') (w_gas(band,i,quad),i=1,m)
            write(10,'(1000e17.9)') (g_gas(band,i,quad),i=1,m)
            write(10,'(1000e17.9)') (k_aero(band,i),i=1,m)
            write(10,'(1000e17.9)') (ka_aero(band,i),i=1,m)
            write(10,'(1000e17.9)') (kd_aero(band,i),i=1,m)
            write(10,'(1000e17.9)') (w_aero(band,i),i=1,m)
            write(10,'(1000e17.9)') (g_aero(band,i),i=1,m)
            write(10,'(1000e17.9)') (k_cloud(band,i),i=1,m)
            write(10,'(1000e17.9)') (ka_cloud(band,i),i=1,m)
            write(10,'(1000e17.9)') (kd_cloud(band,i),i=1,m)
            write(10,'(1000e17.9)') (w_cloud(band,i),i=1,m)
            write(10,'(1000e17.9)') (g_cloud(band,i),i=1,m)
         enddo
      enddo
      close(10)

 666  continue
      if (t.eq.1) then
         write(*,*) 'The data file has not been generated !'
      else
         write(*,*) 'Atmospheric data file successfully generated'
      endif
      
      end






      subroutine test_nan(dat,res)
      implicit none
      integer t1,t2,res
      double precision dat
c
c     Purpose: to test whether the input double precision 'dat'
c     is a NaN or not.
c
c     Input: dat (double precision)
c
c     Output: res
c     res=0: 'dat' has a real value
c     res=1: 'dat' in a NaN
c
      res=0

      if (dat.le.0.0D+0) then
         t1=0
      else
         t1=1
      endif
      if (dat.ge.0.0D+0) then
         t2=0
      else
         t2=1
      endif
      
      if ((t1.eq.1).and.(t2.eq.1)) then
         res=1
      endif
      
      return
      end
