      program make_meshGCM
      implicit none

      include './max.inc'
      include './param.inc'
      integer m,i
      double precision pres_inter(0:Nmax)
      double precision pres_cent(1:Nmax)

c     Purpose: to generate a "meshGCM.txt" file that can be used by KARINE
c     Using a "meshGCM.txt" file is usefull when the user wants to use a precise
c     pressure analysis grid (computation and analysis altitude levels 'e' will
c     be associated to the pressure levels provided in this file). This is for
c     atmospheric applications.
c
c     The user has to enter the number of gas layers, and every pressure level
c     at layers interfaces.
c     Pressure levels at layers centers are also required. Karine will use
c     both interface and center pressures for interpolating interface altitude levels.
c


c     First provide data:

      m=5 ! number of atmospheric layers

c     Now pressure levels at layers INTERFACES.
      pres_inter(0)=1013.0D+2 ! pressure at ground level (Pa)
      pres_inter(1)=900.0D+2 ! pressure at the top of layer 1 (Pa)
      pres_inter(2)=600.0D+2 ! pressure at the top of layer 2 (Pa)
      pres_inter(3)=100.0D+2 ! pressure at the top of layer 3 (Pa)
      pres_inter(4)=10.0D+2 ! pressure at the top of layer 4 (Pa)
      pres_inter(5)=1.0D+2 ! pressure at the top of layer 5 = top of the model (Pa)

c     Pressure levels at layers CENTERS.
      pres_cent(1)=950.0D+2 ! pressure at center of layer 1 (Pa)
      pres_cent(2)=750.0D+2 ! pressure at center of layer 2 (Pa)
      pres_cent(3)=40.0D+2 ! pressure at center of layer 3 (Pa)
      pres_cent(4)=50.0D+2 ! pressure at center of layer 4 (Pa)
      pres_cent(5)=50.0D+2 ! pressure at center of layer 5 (Pa)


c     Next, generate './meshGCM.txt' file
      open(11,file='./meshGCM.txt')
      write(11,*) ' Niveaux de pression du GCM (en Pa):'
      write(11,*) ' i / inter-couches / milieu couches'
      write(11,*)
      do i=0,m-1
         write(11,*) i+1,pres_inter(i),pres_cent(i+1)
      enddo
      write(11,*) m+1,pres_inter(m)
      write(11,*) m+1,0.0D+0 ! This line is used by KARINE; it is mandatory !
      close(11)
      write(*,*) 'File ./meshGCM.txt was succesfully generated'

      end
