c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine abs_gaz(sens,band,quad,DeltaXmax,j,ksi_in,w_in,
     &     alpha_in,B_em,B_abs,alpha_out,ksi_abs,w_abs,ksi_out,
     &     w_out,arret)

      implicit none

c     Cette routine traite l'absorption en maille gazeuse
c
c     Entres :
c       + sens : sens de propagation du rayon
c       + band : numro de la bande troite
c       + quad : numro de la quadrature
c       + DeltaXmax : delta T max dans l'atmosphre
c       + j : numro de la maille de gaz
c       + ksi_in : nergie du rayon  l'entre de la maille
c       + w_in : poids du rayon  l'entre de la maille
c       + alpha_in : angle d'entre dans la maille gazeuse
c       + B_em : luminance du point d'mission du rayon
c     
c     Sorties :
c       + B_abs : luminance du point d'absorption du rayon
c       + alpha_out : angle de sortie de la maille
c       + ksi_abs : nergie absorbe dans la maille
c       + w_abs : poids du rayon  l'absorption
c       + ksi_out : nergie transmise hors de la maille
c       + w_out : poids du rayon  la sortie de la maille
c       + arret : si arret=0, le rayon peut continuer son chemin
c                 si arret=1, le chemin de diffusion a t tronqu
c

      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      include 'options.inc'
      double precision ksi_in,alpha_in,B_em,ksi_abs,ksi_out,z0,ksi_min
      integer sens,j,arret,r_max,p,i,nd,band,quad
      double precision ray(0:DiffMax,2),alpha_out,raf1,tau,tau_out,pdf
      double precision w_absorb,tau1,tau2,alpha,z_abs,B_abs,tau_complet
      double precision w_in,w_out,w_abs,DeltaXmax
c     Debug
      integer t1,t2,r
c     Debug

c     Calcul de la position d'entre dans la maille et de ksi_min
      if (alpha_in.lt.pi/2.D+0) then
         z0=e(j)
      else
         z0=e(j-1)
      endif
      call calc_stop(band,quad,DeltaXmax,B_em,ksi_min)

c     Calcul du trajet optique de diffusion du rayon
      call path(band,quad,j,z0,alpha_in,ksi_in,ksi_min,ray,r_max,nd,
     &     alpha_out,arret)
      call countd(0,j,nd,ray(r_max,2))

c     Calcul de "tau_complet", l'p. opt. d'abs. du chemin optique
      call find_tau_abs(band,quad,j,ray,r_max,tau_complet)
c     Debug
c      if (j.eq.3) then
c         do i=1,r_max
c            write(*,*) 'ray(',i,')=',ray(i,1),ray(i,2)
c         enddo ! i
c      endif
c      write(*,*) 'j=',j,' tau_complet=',tau_complet
c     Debug
c     Debug
c      if (tau_complet.le.0.0D+0) then
c         t1=0
c      else
c         t1=1
c      endif
c      if (tau_complet.ge.0.0D+0) then
c         t2=0
c      else
c         t2=1
c      endif
c      if ((t1.eq.1).and.(t2.eq.1)) then
c         write(*,*) 'r_max:',r_max
c         do r=1,r_max
c            write(*,*) 'ray(',r,',1):',ray(r,1),' ray(',r,',2):'
c     &           ,ray(r,2)
c         enddo
c      endif
c     Debug

c     Calcul de la position d'absorption et de la luminance du point d'absorption
      call pdf_tau_abs(1,tau_complet,raf1,tau_out,pdf)
      w_absorb=1.0D+0/pdf
      
      tau1=0.0D+0
      do p=2,r_max
         call find_tau_abs(band,quad,j,ray,p,tau2)
         if ((tau1.lt.tau_out).and.(tau2.gt.tau_out)) then
            goto 10000
         endif
         tau1=tau2
      enddo
10000 continue
      alpha=dacos((ray(p-1,1)-ray(p,1))/ray(p,2))
      tau=tau_out-tau1
      call find_z1_abs(band,quad,j,ray(p-1,1),alpha,tau,z_abs)
      if (z_abs.lt.0.0D+0) then
         call screen(1107,0,0,0.0D+0,0.0D+0)
         stop
      endif
      
      call find_B(band,quad,z_abs,B_abs)
      
c     Calcul de la partie absorbe et de la partie transmise

      ksi_out=ksi_in*dexp(-tau_complet)
c     Debug
c      if (ksi_out.le.0.0D+0) then
c         t1=0
c      else
c         t1=1
c      endif
c      if (ksi_out.ge.0.0D+0) then
c         t2=0
c      else
c         t2=1
c      endif
c      if ((t1.eq.1).and.(t2.eq.1)) then
c         write(*,*) 'ksi_out:',ksi_out
c         write(*,*) 'tau_complet:',tau_complet
c      endif
c     Debug
      ksi_abs=ksi_in-ksi_out
c     Debug
c      if (j.eq.3) then
c         write(*,*) 'ksi_in=',ksi_in
c         write(*,*) 'ksi_out=',ksi_out
c         write(*,*) 'ksi_abs=',ksi_abs
c      endif
c     Debug
      w_out=w_in
c     Debug
      w_abs=w_in
c      w_abs=dabs(dcos(ang_tir_wall*pi/180.0D+0))
c     Debug

c     Rtablir le sens de propagation du rayon
      if (arret.eq.0) then
         if (alpha_out.le.pi/2.0D+0) then
            sens=0
         else
            sens=1
         endif
      endif

30000 continue
      return
      end
