c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine initz2(j,sens,band,quad,DeltaXmax,w_in,B_em,
     &     alpha_out,w_out,ksi_out,arret)

      implicit none

c     Cette routine cre un chemin de diffusion complet dans la maille d'mission j
c     suivant le processus d'mission numro 2 : choix d'un angle de sortie de maille,
c     gnration d'un chemin de diffusion complet (ou tronqu) jusqu' premire sortie
c     de maille, inversion de ce chemin et choix d'une position d'mission sur le chemin.
c     
c     Entres :
c       + j : numro de la maille gazeuse d'mission
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + band : numro de la bande troite
c       + quad : numro de la quadrature
c       + DeltaXmax : delta T maximum dans l'atmosphre
c       + w_in : poids du rayon  l'mission (choix frquentiel)
c
c     Sortie :
c       + B_em : luminance du point d'mission du rayon
c       + alpha_out : angle de sortie de la maille
c       + w_out : poids de sortie du photon -- combine pdf angulaire et pdf(tau_abs)
c       + ksi_out : nergie restante aprs diffusion dans la maille
c       + arret : si arret=1, le chemin de diffusion a t tronqu ; sinon arret=0
c

      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'
      integer j,r_max,arret,p,i,sens,nd,band,quad
      double precision w_in,DeltaXmax
      double precision alpha_out,w_ang,w_em,ksi_out,w_out,raf1,ksi_min
      double precision tau_abs,alpha,w,pdf,pdf0,B_em,tau_out
      double precision tau_max,ray(0:Diffmax,2),long,z0,z1,alpha0,Bo
      double precision tau,tau1,tau2,tau_complet,wo
      double precision raf,ka,tau_diff,g
c     Debug
      integer r
c     Debug

      ray(0,1)=0.0D+0
      ray(0,2)=0.0D+0

c     Choix d'un angle de sortie de maille et calcul du poids angulaire
      ray(1,1)=e(j-1)
      ray(1,2)=0.0D+0
      ray(2,1)=e(j)
      ray(2,2)=e(j)-e(j-1)
      call find_tau_abs(band,quad,j,ray,2,tau_abs) ! Calcul de l'ep. opt. d'abs. de la maille j
      call find_tau_diff(band,quad,j,ray,2,tau_diff) ! Calcul de l'ep. opt. de diff. de la maille j
c     Dcouverte d'un bug par Maxime / mars 2003
c      call find_g((e(j)-e(j+1))/2.0D+0,g) ! Calcul du g pour la maille j
      call find_g(band,quad,j,(e(j)+e(j-1))/2.0D+0,g) ! Calcul du g pour la maille j
c     Bravo Maxime !
      call pdf_angulaire(1,sens,1,raf1,tau_abs,tau_diff,g,alpha,
     &     pdf0,pdf)            ! Calcul de l'angle et de son pdf
      w_ang=pdf0/pdf
      
c     Dtermination de l'altitude de dpart du rayon sur le segment
      if (alpha.lt.pi/2.D+0) then ! Calcul de z0, l'altitude de dpart du segment
         z0=e(j)
      else
         z0=e(j-1)
      endif
      alpha_out=pi-alpha

c     Calcul de la luminance du pt de dpart et le ksi_min associ
      call find_B(band,quad,z0,Bo)
      call calc_stop(band,quad,DeltaXmax,Bo,ksi_min)

c     Calcul du chemin de diffusion jusqu' premire sortie de la maille j
      call path(band,quad,j,z0,alpha,1.0D+0,ksi_min,ray,r_max,
     &     nd,raf,arret)

c     Rtablir le sens de propagation du rayon
      if (arret.eq.0) then
         if (alpha_out.le.pi/2.0D+0) then
            sens=0
         else
            sens=1
         endif
      endif

      call countd(0,j,nd,ray(r_max,2))
      
c     Debug
c      write(*,*) 'new'
c      do r=1,r_max
c         write(*,*) 'ray(',r,',1):',ray(r,1),' ray(',r,',2):',ray(r,2)
c      enddo
c     Debug

      call find_tau_abs(band,quad,j,ray,r_max,tau_complet)
      
c     Calcul de l'altitude o le photon sera mis
      call pdf_tau_abs(1,tau_complet,raf1,tau_out,pdf)
      w_em=1.D+0/pdf
c     A partir d'ici tau_out est l'ep. opt. d'abs. entre le point d'mission
c     et la fin du chemin

      tau1=0.0D+0
      do p=2,r_max
         call find_tau_abs(band,quad,j,ray,p,tau2)
         if ((tau1.le.tau_out).and.(tau2.gt.tau_out)) then
            goto 10000
         endif
         tau1=tau2
      enddo
10000 continue
      alpha=dacos(dble(real((ray(p-1,1)-ray(p,1))/
     &     (ray(p,2)-ray(p-1,2)))))
      tau=tau_out-tau1
      call find_z1_abs(band,quad,j,ray(p-1,1),alpha,tau,z1)
      if (z1.lt.0.0D+0) then
         call screen(1108,0,0,0.0D+0,0.0D+0)
         stop
      endif

c     Calcul de la luminance du point d'mission
      call find_B(band,quad,z1,B_em)

c     Calcul de l'absorption sur la longueur du chemin gnr
      ksi_out=dexp(-tau_out)
      
c     Calcul du poids total du photon  la sortie de la maille d'mission
      if (tirpg.eq.1) then    
         wo=1.0D+0
      else
         wo=pi
      endif
c     Debug
c      if (w_ang.ne.1.0D+0) then
c         write(*,*) 'w_ang=',w_ang
c         stop
c      endif
c     Debug
      w_out=w_in*wo*w_ang*w_em

30000 continue
      return
      end
