c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine ksi_matrix(r,band,T0,pne,ksi)
      implicit none
      include 'max.inc'
      include 'data.inc'
      include 'param.inc'
c
c     Purpose: to compute the matrix of optico-geometric coefficients ksi
c
c     ksi(i,j) has no unit.
c     In order to re-compute a NER matrix over a narrowband 'band',
c     one has to multiply ksi(i,j,band) by pi*(B(i,band)-B(j,band)) using
c     the same convention, i.e. B(i,band) and B(j,band) in W/m/sr,
c     which gives psi(i,j,band) in W/m.
c
c     Input:
c       + r:
c        - r=0: use non-recombined grid
c        - r=1: use recombined grid
c       + band: band index
c       + T0: temperature profile
c        - T0(0): temperature at the bottom of layer 1
c        - T0(i), i=1,Nlay: temperature of each layer i
c        - T0(Nlay+1): temperature at the top of layer Nlay
c       + pne: NER matrix (W/m^2)
c
c     Ouput:
c       + ksi: ksi matrix (no unit)
c
c     I+O
      integer r
      double precision T0(0:Nmax+1)
      double precision pne(0:Nmax+1,0:Nmax+1)
      double precision ksi(0:Nmax+1,0:Nmax+1)
c     temp
      integer i,j,band,res,nlay
      double precision Ti,Tj
      double precision Bi,Bj
      double precision ksi_min
      parameter(ksi_min=1.0D-60)
c     functions
      double precision Planck,planck_int_wavlng
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine ksi_matrix'

c     Purpose: to compute the matrix of 
c     ksi(i,j,band)=ner(i,j,band)/{pi*[B(i,band)-B(j,band)]}
c     for a given band
c
      if (r.eq.0) then
         nlay=m
      else
         nlay=m-2
      endif

c     Computation of ksi
      do i=0,nlay+1
c     pne(i,j) is given in W/m

c     Debug
         do j=0,nlay+1
            call test_nan(pne(i,j),res)
            if (res.eq.1) then
               call screen(1131,i,j,dble(band),pne(i,j))
               stop
            endif
         enddo
c     Debug

         Bi=planck_int_wavlng(lambda_lo(band),lambda_hi(band),T0(i)) ! W/m/sr
         do j=0,nlay+1
            Bj=planck_int_wavlng(lambda_lo(band),lambda_hi(band),T0(j)) ! W/m/sr
            if ((j.eq.i).or.(Bi.eq.Bj)) then
               ksi(i,j)=0.0D+0
            else
               ksi(i,j)=pne(i,j)/(pi*(Bi-Bj)) ! [no unit]
               if (dabs(ksi(i,j)).lt.ksi_min) then
                  ksi(i,j)=0.0D+0
               endif
            endif
         enddo ! j
      enddo ! i

      return
      end
