c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine wall(gen,sens,band,quad,j,ksi_min,alpha_in,w_in,
     &     ksi_in,alpha_out,B_wall,w_out,w_abs,ksi_abs,ksi_out,arret)

      implicit none

c     Routine de traitement des actions lies  la paroi
c     Si gen=1, on traite les problmes d'mission
c     Si gen=0, on traite les problmes d'absorption et de rflexion
c
c     Entres :
c       + gen : cf. ci-dessus
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + band : numro de la bande troite
c       + quad : numro de la quadrature
c       + j : indice de la maille reprsentant la paroi (0 ou m+1)
c       + ksi_min : nergie minimale du rayon en dessous de laquelle on considre qu'il est
c                   entirement absorb
c       + alpha_in : angle d'arrive sur la paroi s'il s'agit de traiter une rflexion
c       + w_in : poids d'arrive du rayon
c       + ksi_in : nergie du rayon (pour une rflexion ou une absorption)
c
c     Sorties :
c       + alpha_out : angle de dpart du rayon  la paroi
c       + B_wall : luminance de la paroi en question (0 ou m+1)
c       + w_out : poids du rayon au dpart de la paroi
c       + w_abs : poids du rayon  l'absorption
c       + ksi_abs : nergie du rayon qui est absorbe par la paroi
c       + ksi_out : nergie du rayon au dpart de la paroi
c       + arret : si arret=1, le rayon a t absorb par la paroi entirement
c

      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      include 'options.inc'

      integer j,gen,arret,sens
      double precision alpha_in,w_in,ksi_in,ksi_min,w_abs,w0,wo
      double precision alpha_out,w_out,ksi_out,ksi_abs
      double precision raf1,raf2,alpha,pdf0,pdf,B_wall,z
      double precision Xsol,Xespace,DeltaXmax
      double precision tau_abs_atm,tau_diff_atm,g,ray(0:DiffMax,2)
      double precision gint,tdint
      double precision tau_abs_int,tau_diff_int
      integer ptl,profil,p,Nb,Nq,band,quad
      double precision Planck
      
      arret=0
      w0=w_in
      w_abs=w_in
c     Debug
c         write(*,*) 'w_abs=',w_abs
c     Debug
      if (j.eq.0) then
         if (udif.eq.1) then
            B_wall=Planck(Tsol,lambda_lo(band),lambda_hi(band))
         else
            B_wall=udif_sol(band)*dabs(dcos(ang_tir_wall*pi/180.0D+0))
c     Debug
c            write(*,*) 'B_wall=',B_wall
c            write(*,*) 'udif_sol(',band,')=',udif_sol(band)
c     Debug
         endif
c     Debug
c         write(*,*) 'lambda=',(lambda_lo(band)+lambda_hi(band))/2.0D+0,
c     &        ' Tsol=',Tsol,' B=',B_wall
c     Debug
      else if (j.eq.m+1) then
c     28/01/2005 Bravo  David pour avoir permis d'identifier ce
c     bug majeur (qui n'tait jamais apparu avant parce que je me
c     suis toujours content d'avoir Tesp=0)
c         B_wall=Tesp
         if (udif.eq.1) then
            B_wall=Planck(Tesp,lambda_lo(band),lambda_hi(band))
         else
            B_wall=udif_esp(band)*dabs(dcos(ang_tir_wall*pi/180.0D+0))
c     Debug
c            write(*,*) 'B_wall=',B_wall
c     Debug
         endif
c     
      else
         call screen(1105,0,0,0.0D+0,0.0D+0)
      endif

      call calc_tau_abs_mlay(band,quad,1,m,2,tau_abs_atm)
      call calc_tau_diff_mlay(band,quad,1,m,2,tau_diff_atm)
      
c     Calcul d'un "g quivalent" pour toute l'atmosphre
      g=0.0D+0
      do p=1,m
         ray(1,1)=e(p-1)
         ray(1,2)=0.0D+0
         ray(2,1)=e(p)
         ray(2,2)=e(p)-e(p-1)
         call find_tau_diff(band,quad,p,ray,2,tdint)
         call find_g(band,quad,p,(e(p)+e(p-1))/2.0D+0,gint)
         g=g+gint*tdint
      enddo
      g=g/tau_diff_atm

      if (gen.eq.1) then ! Emission d'un rayon depuis une paroi
c     Tirage d'un angle  la paroi
         call pdf_angulaire(1,sens,0,raf1,tau_abs_atm,tau_diff_atm,g,
     &        alpha,pdf0,pdf)
         if (j.eq.0) then ! pour le sol on doit tirer vers le haut
            alpha_out=pi-alpha
         else ! pour l'espace on doit tirer vers le bas
            alpha_out=alpha
         endif
         if (tirpp.eq.1) then
c     Je change la valeur de wo dans ce cas l pour avoir un rsultat
c     compatible pour les simulations atmosphriques 3D
c            wo=1.0D+0
            if (udif.eq.1) then
               wo=pi
            else
               wo=1.0D+0
            endif
         else
            if (udif.eq.1) then
               wo=pi
            else
               wo=1.0D+0
            endif
         endif
         w_out=w0*wo*pdf0/pdf
         if (j.eq.0) then
            ksi_out=em_sol(band)
         else
            ksi_out=em_esp(band)
         endif
      else if (gen.eq.0) then   ! Rflexion d'un rayon sur une paroi
c     Tirage d'un angle  la paroi
         if (ref.eq.1) then ! Si la rflexion est spculaire
            call pdf_ang_spec(alpha_in,alpha_out,pdf)
c     Debug
c            write(*,*) 'PDF_ANGULARE=',pdf
c     Debug
         else if (ref.eq.2) then ! Si la rflexion est isotrope
            call pdf_ang_iso(1,sens,0,alpha,pdf)
            if (j.eq.0) then ! pour le sol on doit tirer vers le haut
               alpha_out=pi-alpha
            else ! pour l'espace on doit tirer vers le bas
               alpha_out=alpha
            endif
         else
            call screen(1005,0,0,0.0D+0,0.0D+0)
            stop
         endif
         if (udif.eq.1) then
            call pdf_ang_lambert(0,sens,0,alpha,pdf0)
         else
            pdf0=1.0D+0
         endif
         w_out=w0*pdf0/pdf
c     Debug
c         write(*,*) 'w_out=',w_out
c         write(*,*) 'w0=',w0
c         write(*,*) 'pdf0=',pdf0
c         write(*,*) 'pdf=',pdf
c         write(*,*) 'alpha=',alpha
c         write(*,*) 'dcos(',alpha,')/pi=',dcos(alpha)/pi
c     Debug
         if (j.eq.0) then
            ksi_abs=ksi_in*em_sol(band)
            ksi_out=ksi_in*(1.0D+0-em_sol(band))
         else if (j.eq.m+1) then
            ksi_abs=ksi_in*em_esp(band)
            ksi_out=ksi_in*(1.0D+0-em_esp(band))
         else
            call screen(1101,0,0,0.0D+0,0.0D+0)
            stop
         endif
      endif

      if (ksi_out.le.ksi_min) then ! Si le rayon a t totalement absorb
         arret=1
      endif

c     Ne pas oublier de changer le sens de propagation dans le cas
c     d'une rflexion  la paroi si le rayon n'est pas arrt
      if ((gen.eq.0).and.(arret.eq.0)) then
         if (sens.eq.0) then
            sens=1
         else
            sens=0
         endif
      endif

30000 continue 
      return
      end
